@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use base qw(Catalyst::Controller::CRUD);
 
-our $VERSION = '0.19';
+our $VERSION = '0.20';
 
 =head1 NAME
 
@@ -68,13 +68,28 @@ None by default.
 
 This method returns model object having $id.
 
+Triggers:
+
+ $self->call_trigger( 'get_model_after', $c, $hash );
+
 =cut
 
 sub get_model {
     my ( $this, $c, $self, $id ) = @_;
-    my $setting = $self->setting($c);
-    my $primary = $setting->{primary};
+
+    my $name    = $self->setting($c)->{name};
+    my $primary = $self->setting($c)->{primary};
     my $model   = $c->model( $self->setting($c)->{model} )->retrieve( $primary => $id );
+
+    if (defined $model) {
+        my $hash    = $model->toHashRef;
+        $self->call_trigger( 'get_model_after', $c, $hash );
+        $c->stash->{ $name } = $hash;
+    } else {
+        $c->res->status(404);
+        $c->res->body("404 Not Found\n");
+    }
+
     return $model;
 }
 
@@ -82,26 +97,23 @@ sub get_model {
 
 This method returns model objects.
 
-Triggers:
-
- $self->call_trigger( 'list_where_make_phrase', $c, $where );
- $self->call_trigger( 'list_order_make_phrase', $c, $order );
-
 =cut
 
 sub get_models {
     my ( $this, $c, $self ) = @_;
 
-    my $where = { disable => 0 };
-    $self->call_trigger( 'list_where_make_phrase', $c, $where );
+    my $name    = $self->setting($c)->{name};
+    my $primary = $self->setting($c)->{primary};
+    my $where   = $c->stash->{$name}->{where} ? $c->stash->{$name}->{where} : { disable => 0 };
+    my $order   = $c->stash->{$name}->{order} ? $c->stash->{$name}->{order} : { order_by => $primary };
 
-    my $order = { order_by => $self->setting($c)->{primary} };
-    $self->call_trigger( 'list_order_make_phrase', $c, $order );
-
-    my @models  = $c->model( $self->setting($c)->{model} )->search_where( $where, $order );
+    my @models = $c->model( $self->setting($c)->{model} )->search_where( $where, $order );
     my @result;
     foreach (@models) {
-        push(@result, $_->toHashRef);
+        my $hash = $_->toHashRef;
+        $self->call_trigger( 'get_model_after', $c, $hash );
+        $c->stash->{ $name . '_' . $primary . 's' }->{ $hash->{$primary} } = $hash;
+        push( @result, $hash );
     }
     return \@result;
 }
